/*Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana

This file is part of Plataforma Integrada MEC.

Plataforma Integrada MEC is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Plataforma Integrada MEC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with Plataforma Integrada MEC.  If not, see <http://www.gnu.org/licenses/>.*/

import React, { useState, useEffect, useContext } from 'react';
//imports material ui components
import { Typography, Grid } from '@material-ui/core';
import CircularProgress from '@material-ui/core/CircularProgress';
import CardContent from "@material-ui/core/CardContent";
import CardAction from '@material-ui/core/CardActions';
import ListRoundedIcon from '@material-ui/icons/ListRounded';
import SaveIcon from '@material-ui/icons/Save';
import MenuItem from "@material-ui/core/MenuItem";
//imports local files
import SnackBar from '../../../../Components/SnackbarComponent';
import { getRequest, putRequest } from '../../../../Components/HelperFunctions/getAxiosConfig'
import LoadingSpinner from '../../../../Components/LoadingSpinner';
import { EditFilter, GetAData } from '../../../Filters';
import { StyledCard } from "../../../Components/Styles/DataCard";
import { Store } from 'Store'
import FormInput from "Components/FormInput.js"
import StyledButton from '../Button';
//routers
import { Link } from 'react-router-dom';
import ClassicEditor from "@ckeditor/ckeditor5-build-classic"
import { CKEditor } from '@ckeditor/ckeditor5-react';

const EditCollection = () => {

    const { state } = useContext(Store);

    const urlParams = new URLSearchParams(window.location.search);
    const id = urlParams.get("collection");

    const [error, setError] = useState(null); //Necessary to consult the API, catch errors
    const [isLoaded, setIsLoaded] = useState(false); //Necessary to consult the API, wait until complete

    const [isLoading, setIsLoading] = useState(false);
    const [name, setName] = useState('')
    const [privacy, setPrivacy] = useState('')
    const [description, setDescription] = useState('');

    const [errorInName, setErrorInName] = useState({
        error: false,
        message: ''
    })

    const [snackInfo, setSnackInfo] = useState({
        message: '',
        icon: '',
        open: false,
        color: '',
    })

    const NameHandler = (e) => {
        if (errorInName.error) {
            setErrorInName({
                error: false,
                message: ''
            })
        }
        setName(e.target.value)
    }

    // Fields
    const fields = [
        {
            label: 'ID',
            value: id,
            default: true,
            type: 'text'
        },
        {
            label: 'Nome',
            value: name,
            error: errorInName.error,
            errorMessage: errorInName.message,
            onChange: (event) => NameHandler(event),
            default: false,
            type: 'text',
        },
    ]

    const privacyOptions = [
        {
            value: "private",
            label: "Privado",
        },
        {
            value: "public",
            label: "Público",
        },
    ];

    //handle change of privacy
    const handleChange = (e) => {
        const value = e.target.value;
        setPrivacy(value);
    };

    // Handle snack infos
    const HandleSnack = (message, state, icon, color) => {
        setSnackInfo({
            message: message,
            icon: icon,
            open: state,
            color: color
        })
    }

    const onSubmit = async () => {
        setIsLoading(true)
        const api = EditFilter('collections', id)
        let body = {
            "collection": {
                "name": name,
                "privacy": privacy,
                "description": description
            }
        }
        putRequest(
            api,
            body,
            (data) => {
                if (data.errors) {
                    HandleSnack(`${data.errors[0]}`, true, 'warning', '#FA8072')
                }
                else {
                    HandleSnack('A Coleção foi alterada com sucesso', true, 'success', '#228B22')
                }
                setIsLoading(false)
            },
            (error) => {
                HandleSnack('Ocorreu algum erro', true, 'warning', '#FA8072')
                setIsLoading(false)
            }
        )
    }

    useEffect(() => {
        getRequest(
            GetAData("collections", id),
            (data, header) => {
                setIsLoaded(true);
                setError(false);
                setName(data.name)
                setPrivacy(data.privacy)
                setDescription(data.description)
            },
            (error) => {
                setIsLoaded(true);
                setError(true);
            }
        )
    }, []);

    if (error) {
        return <div> Houve um erro... </div>
    } else if (!isLoaded) {
        return <LoadingSpinner text="Carregando..." />
    } else {
        return (
            <StyledCard contrast={state.contrast}>
                <SnackBar
                    severity={snackInfo.icon}
                    text={snackInfo.message}
                    snackbarOpen={snackInfo.open}
                    color={snackInfo.color}
                    handleClose={() => setSnackInfo({
                        message: '',
                        icon: '',
                        open: false,
                        color: ''
                    })}
                />
                <CardContent>
                    <Grid container direction='row' justify='space-between' xs={12} alignItems="center">
                        <Grid item>
                            <Typography variant='h4'>
                                {name}
                            </Typography>
                        </Grid>
                        <Grid item>
                            <Link style={{ textDecoration: 'none' }} to={'/admin/Collections'}>
                                <StyledButton
                                    icon={<ListRoundedIcon />}
                                    variant='outlined'
                                    color='primary'
                                    text="Listar"
                                />
                            </Link>
                        </Grid>
                    </Grid>

                    <div style={{ height: '1em' }}></div>

                    <form style={{ display: 'flex', flexDirection: 'column' }}>
                        {fields.map((field, index) => (
                            <FormInput
                                contrast={state.contrast}
                                key={index}
                                required={field.required}
                                disableField={field.default}
                                error={field.error}
                                help={field.error ? field.errorMessage : ''}
                                style={{ marginBottom: '1em' }}
                                placeholder={field.label}
                                value={field.value}
                                handleChange={field.onChange}
                                inputType="search"
                                multi={true}
                            />
                        ))}
                        <FormInput
                            contrast={state.contrast}
                            selectable={true}
                            placeholder="Privacidade"
                            value={privacy ? privacy : ""}
                            handleChange={handleChange}
                            items={privacyOptions.map((option, index) => (
                                <MenuItem
                                    key={option.value}
                                    value={option.value}
                                    className={`${state.contrast}BackColor ${state.contrast}LinkColor`}
                                >
                                    {
                                        option.value
                                    }
                                </MenuItem>
                            ))}
                        />
                        <div style={{color: "#666"}}>
                            <CKEditor
                                editor={ClassicEditor}
                                data={description}
                                onBlur={(event, editor) => {
                                    const data = editor.getData();
                                    setDescription(data)
                                }}
                            />
                        </div>
                    </form>
                </CardContent>

                <CardAction>
                    <StyledButton
                        variant="contained"
                        color="primary"
                        disabled={isLoading}
                        icon={isLoading ? null : <SaveIcon />}
                        onClick={onSubmit}
                        text={isLoading ? <CircularProgress size={24} /> : 'Salvar'}
                    />
                </CardAction>
            </StyledCard>
        )
    }
}

export default EditCollection;